'###################################################################################
' Copyright (c) 2013 bent.schrader@googlemail.com
' All Rights Reserved
'
' Set ACL rights for profile or user folders

' Declare Program Variables
Dim WshShell,oExec,objArgs,strMessage,strVar,objFSO,objFolder,strUser,strScriptName,strFolder,strPath,strCommand
Dim objLog,objADUser,i,j,x,objWMIService,Choice,strForce
Dim objUser

' Initialize Constants & Variables
Const strAdminGroup = "*S-1-5-32-544"
Const strCreatorOwner = "*S-1-3-0"
Const scriptVersion = "v1.1"

strForce = False
i = 0
x = 0

Function ErrorMessage(strMessage)
	WScript.Echo "ERROR: " & strMessage
	WScript.Quit(1)
End Function

Function Usage
	WScript.Echo vbCrlf & "USAGE:" & vbCrlf & "cscript /nologo " & strScriptName &".vbs <full-folder-path> <force>"
	WScript.Quit(0)
End Function

Function Output(strString)
	Wscript.Echo strString
	objLog.WriteLine strString
End Function

Function Execute(strCommand)
	objLog.WriteLine " [" & strCommand & "]"
	Set oExec = WshShell.Exec(strCommand)
	Do While Not (oExec.StdOut.AtEndOfStream)
		objLog.WriteLine(oExec.StdOut.ReadLine())
	Loop
End Function

strScriptName = left(Wscript.ScriptName, len(Wscript.ScriptName)-4)   ' Pick up Script Name
Set WshShell = CreateObject("WScript.Shell")
Set objEnv = WshShell.Environment("Process")
Set objFSO = CreateObject("Scripting.FileSystemObject")

strPath = Wscript.ScriptFullName
Set objFile = objFSO.GetFile(strPath)
strLogFile = objFSO.GetParentFolderName(objFile) & "\set-acl.log"
Set objLog = objFSO.CreateTextFile(strLogFile)

' Validate Input Parameters
Set objArgs = WScript.Arguments
select Case objArgs.Count
	Case 1
		strFolder = objArgs(0)
	Case 2
		strFolder = objArgs(0)
		strForce = objArgs(1)
	Case Else
		Usage
End Select

Time1 = Time()

'Modify root folder	
Output vbCrlf & "------------------------------------------------"
Output " Set ACL Tool for user and profile folders " & scriptVersion
Output "------------------------------------------------" & vbCrlf
Output " Script started at: " & now & vbCrlf

'Check existence
If objFSO.FolderExists(strFolder) Then
    Set objFolder = objFSO.GetFolder(strFolder)
Else
    ErrorMessage ("The folder [" & Ucase(strFolder) & "] does not exist.")
End If

Output " Processing folder [" & Ucase(strFolder) & "] and " & objFolder.SubFolders.Count & " subfolders." & vbCrlf

If Lcase(strForce) <> "force" Then
	Choice=MsgBox("Folder [" & Ucase(strFolder) & "] does have " & objFolder.SubFolders.Count & " subfolders.",1,Ucase(strScriptName) & " " & scriptVersion)
	If Choice=vbCancel Then WScript.Quit
End If

'Get domain SID
Output "[Step 1]: Getting domain information" & vbCrlf
Set objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & Lcase(objEnv("USERDNSDOMAIN")) & "\root\MicrosoftActiveDirectory")
Set colDomainInfo = objWMIService.ExecQuery("Select * from Microsoft_LocalDomainInfo")

For each objDomain in colDomainInfo
    strDomain = objDomain.DNSName
	strSID = objDomain.SID
Next

If Lcase(strForce) <> "force" Then
	Choice=MsgBox("Domain [" & Lcase(strDomain) & "] does have SID [" & strSID & "]",1,Ucase(strScriptName) & " " & scriptVersion)
	If Choice=vbCancel Then WScript.Quit
End If

Output " Using Active Directory domain [" & Lcase(strDomain) & "] for user matching"
Output " Domain [" & Lcase(strDomain) & "] does have SID [" & strSID & "]" & vbCrlf

Output "[Step 2]: Reset ACls for folder [" & Ucase(strFolder) & "]" & vbCrlf

If Lcase(strForce) <> "force" Then
	Choice=MsgBox("Reset ACLs for folder [" & Ucase(strFolder) & "]",1,Ucase(strScriptName) & " " & scriptVersion)
	If Choice=vbCancel Then WScript.Quit
End If

Output " Take ownership for group Administrators for folder [" & Ucase(strFolder) & "]"
Execute "takeown /f " & strFolder & " /a"
Output " Set ownership for group Domain-Admins for folder [" & Ucase(strFolder) & "]"
Execute "icacls " & strFolder & " /setowner *" & strSID & "-512"
Output " Getting existing ACLs for folder [" & Ucase(strFolder) & "]"
Execute "icacls " & strFolder
Output " Remove any existing inheritance for folder [" & Ucase(strFolder) & "]"
Execute "icacls " & strFolder & " /inheritance:r"
Output " Add SYSTEM account with full access to folder [" & Ucase(strFolder) & "]"
Execute "icacls " & strFolder & " /grant:r SYSTEM:(OI)(CI)F"
Output " Add Domain-Admins with full access to folder [" & Ucase(strFolder) & "]"
Execute "icacls " & strFolder & " /grant:r *" & strSID & "-512:(OI)(CI)F"
Output " Add CREATOR-OWNER with special permissions to folder [" & Ucase(strFolder) & "]"
Execute "icacls " & strFolder & " /grant:r " & strCreatorOwner & ":(OI)(CI)(IO)F"
Output " Add Domain-User with special permissions to folder [" & Ucase(strFolder) & "]"
Execute "icacls " & strFolder & " /grant:r *" & strSID & "-513:(rd,ra,rea,ad,wa,wea,rc,wo)"
Output " Remove group Administrators from folder [" & Ucase(strFolder) & "]" 
Execute "icacls " & strFolder & " /remove:g " & strAdminGroup

'Connect to Active Directory
Output vbCrlf & "[Step 3]: Connect to Active Directory and query all user objects" 

Set Conn = createobject("ADODB.Connection")
Set iAdRootDSE = GetObject("LDAP://RootDSE")	
strDefaultNamingContext = iAdRootDSE.Get("defaultNamingContext")	
Conn.Provider = "ADsDSOObject"	
Conn.Open "ADs Provider"	

Output vbCrlf & " Using AD Default Naming Context [" & strDefaultNamingContext & "]"

strQueryDL = "<LDAP://" & strDefaultNamingContext & ">;(&(objectCategory=person)(objectClass=user));adspath"
set objCmd = createobject("ADODB.Command")	
objCmd.ActiveConnection = Conn	
objCmd.Properties("SearchScope") = 2 
objCmd.Properties("Page Size") = 100 	
	
objCmd.CommandText = strQueryDL
Set objRs = objCmd.Execute	

objADUser = Array(" ")
While Not objRS.eof
	ReDim Preserve objADUser(UBound(objADUser) + 1)
	objADUser(UBound(objADUser)) = objRS.Fields("adspath")
	objRS.MoveNext
Wend

If UBound(objADUser) <= 1 Then 
	ErrorMessage ("No user objects found in domain [" & Lcase(strDomain) & "]")
Else 
	Output " " & UBound(objADUser) & " user objects found in domain [" & Lcase(strDomain) & "]"
End If

If Lcase(strForce) <> "force" Then
	Choice=MsgBox(UBound(objADUser) & " users found in domain [" & Lcase(strDomain) & "]",1,Ucase(strScriptName) & " " & scriptVersion)
	If Choice=vbCancel Then WScript.Quit
End If

'Routine for subfolders
Output vbCrlf & "[Step 4]: Processing ACLs of subfolders for folder [" & Ucase(strFolder) & "]" 

For Each objSubFolder in objFolder.SubFolders
	
	i = i + 1
	
	If Ucase(Right(objSubFolder.Name,3)) = ".V2" Then 
		strUser = Mid(objSubFolder.Name,1,(Len(objSubFolder.Name)-3))
	Else
		strUser = objSubFolder.Name
	End If	
	
	Output vbCrlf & "#" & i & " Reset ACLs for subfolder [" & Ucase(objFolder.Path) & "\" & Ucase(objSubFolder.Name) & "]" & vbCrlf

	bFlag = False
	
	For j = 1 To UBound(objADUser)
		
		Set objUser = GetObject (objADUser(j))
		objUser.GetInfo
		If Lcase(objUser.Get("sAMAccountName")) = Lcase(strUser) Then 
		
		If Lcase(strForce) <> "force" Then
			Choice=MsgBox("Subfolder: [" & strUser & "] matches AD user: " & vbCrlf & objADUser(j),1,Ucase(strScriptName) & " " & scriptVersion)
			If Choice=vbCancel Then WScript.Quit
		End If
		
			Output " Found user [" & strUser & "] in AD (" & objADUser(j) & ")"
			Output vbCrlf & " Take ownership for group Administrators for folder [" & Ucase(objFolder.Path) & "\" & Ucase(objSubFolder.Name) & "]"
			Execute "takeown /f " & objFolder.Path & "\" & objSubFolder.Name & " /a /r /d j"
			Output " Grant full access to group Administrators for subfolders and files in [" & Ucase(objFolder.Path) & "\" & Ucase(objSubFolder.Name) & "]"
			Execute "icacls " & objFolder.Path & "\" & objSubFolder.Name & " /grant:r " & strAdminGroup & ":(OI)(CI)F /t /c /q"
			Output " Deactivate inheritance in " & objFolder.Path & "\" & objSubFolder.Name 
			Execute "icacls " & objFolder.Path & "\" & objSubFolder.Name & " /inheritance:r"
			Output " Set ownership to user [" & strUser & "@" & strDomain & "] for subfolders and files in [" & Ucase(objFolder.Path) & "\" & Ucase(objSubFolder.Name) & "]"
			Execute "icacls " & objFolder.Path & "\" & objSubFolder.Name & " /setowner " & strUser & "@" & strDomain & " /t /c /q"				
			Output " Activate inheritance in " & objFolder.Path & "\" & objSubFolder.Name 
			Execute "icacls " & objFolder.Path & "\" & objSubFolder.Name & " /inheritance:e"				
			Output " Remove the SID of Administrators from all subfolders and files in [" & Ucase(objFolder.Path) & "\" & Ucase(objSubFolder.Name) & "]" 
			Execute "icacls " & objFolder.Path & "\" & objSubFolder.Name & " /remove:g " & strAdminGroup & " /t /c /q"
			Output " Reset the inheritance in [" & Ucase(objFolder.Path) & "\" & Ucase(objSubFolder.Name) & "]" 
			Execute "icacls " & objFolder.Path & "\" & objSubFolder.Name & " /reset /t /c /q"
			
			bFlag = True
	
			Exit For	
		End If
	Next
	
	If Not bFlag Then
			Output " No matching user found in domain [" & Lcase(strDomain) & "] - skip folder [" & Ucase(objFolder.Path) & "\" & Ucase(objSubFolder.Name) & "]"
			x = x + 1
			
			If Lcase(strForce) <> "force" Then
				Choice=MsgBox("No AD user matches for subfolder [" & objSubFolder.Name & "]",1,Ucase(strScriptName) & " " & scriptVersion)
				If Choice=vbCancel Then WScript.Quit
			End If
	End If
		
Next

Output vbCrlf & "[Finished] " & now & ", running time: " & Int(Round((Time() - Time1)*100000)) & " seconds"
Output " " & i & " subfolder(s), " & i-x & " folder(s) processed, " & x & " folder(s) skipped"

Wscript.Quit (0)